==> org-lsst-ccs-subsystem-fcs-main-5.4.4-sources.jar: org/lsst/ccs/subsystems/fcs/common/EPOSControllerForCarousel.java <==
package org.lsst.ccs.subsystems.fcs.common;

import org.lsst.ccs.command.annotations.Command;
import static org.lsst.ccs.subsystems.fcs.EPOSEnumerations.EposMode.HOMING;
import static org.lsst.ccs.subsystems.fcs.EPOSEnumerations.Parameter.HomePosition;
import static org.lsst.ccs.subsystems.fcs.EPOSEnumerations.Parameter.HomingMethod;
import static org.lsst.ccs.subsystems.fcs.EPOSEnumerations.Parameter.PositionSensorType;
import static org.lsst.ccs.subsystems.fcs.FCSCst.FCSLOG;
import static org.lsst.ccs.subsystems.fcs.FcsEnumerations.FcsAlert.HARDWARE_ERROR;
import org.lsst.ccs.subsystems.fcs.errors.FcsHardwareException;
import org.lsst.ccs.subsystems.fcs.utils.FcsUtils;

/**
 *
 * @author virieux
 */
public interface EPOSControllerForCarousel extends EPOSController {

    int getPositionSensorType();

    void setPositionSensorType(int pst);

    /**
     * For carousel, status word is read by PDO so no need to read by SDO.
     * During actions, read by PDO are done every 250ms.
     *
     * @return status word
     */
    @Override
    default boolean isTargetReached() {
        /* target is reached when bit10 of status word is 1 */
        return ((getStatusWord() >> 10) & 1) == 1;
    }


    /**
     * Set PositionSensorType to Absolute encoder SSI (value=4)
     */
    @Command(type = Command.CommandType.ACTION, level = Command.ENGINEERING1, description = "Set PositionSensorType to Absolute encoder SSI (value is 4)")
    default void setPositionSensorTypeEncoderSSI() {
        FCSLOG.info(getName() + " setting PositionSensorType Absolute encoder SSI");
        goToSwitchOnDisabled();
        doSetEncoderSSI();
    }

    default void doSetEncoderSSI() {
        writeParameter(PositionSensorType, 4);
        setPositionSensorType(4);
        /*check that position sensor type is SSI encoder. */
        checkPositionSensorType(4);
        int ssiPosition = readSSIPosition();
        if (Math.abs(ssiPosition) <= 1) {
            /* there is a bug in the controller in reading SSI position*/
 /* switch back to Incremental Encoder */
            this.raiseWarning(HARDWARE_ERROR, " after setPositionSensorTypeEncoderSSI, ssiPosition = "
                    + ssiPosition + " we go back to TypeSinusIncrementalEncoder", getName());
            doSetSinusIncrementalEncoder();
            /* and then back again to Encoder SSI */
            doSetEncoderSSI();
        }
    }

    /**
     * Set PositionSensorType to Sinus Incremental Encoder(value=8)
     */
    @Command(type = Command.CommandType.ACTION, level = Command.ENGINEERING1, description = "Set PositionSensorType to Sinus Incremental Encoder(value is 8)")
    default void setPositionSensorTypeSinusIncrementalEncoder() {
        FCSLOG.info(getName() + " setting PositionSensorType Sinus Incremental Encoder");
        goToSwitchOnDisabled();
        doSetSinusIncrementalEncoder();
    }

    default void doSetSinusIncrementalEncoder() {
        writeParameter(PositionSensorType, 8);
        setPositionSensorType(8);
        /*check that position sensor type is sinus incremental encoder. */
        checkPositionSensorType(8);
    }

    default void checkPositionSensorType(int sensortype) {
        long timeoutMillis = 500;
        final long timeStart = System.currentTimeMillis();
        long duration = 0;
        boolean ok = false;
        while (!ok && duration <= timeoutMillis) {
            duration = System.currentTimeMillis() - timeStart;
            FcsUtils.sleep(2, getName());
            ok = (sensortype == readParameter(PositionSensorType));
        }
        if (!ok) {
            String msg = getName() + " couldn't change position sensor type during time allocated of " + timeoutMillis + " ms";
            FCSLOG.error(msg);
            throw new FcsHardwareException(msg);
        }
        setPositionSensorType(sensortype);
    }

    /**
     *
     * @param position
     */
    @Command(type = Command.CommandType.ACTION, level = Command.ENGINEERING3, description = "Define the actual position as position given as argument.")
    @Override
    default void defineAbsolutePosition(int position) {
        FCSLOG.debug(getName() + " Defining Absolute Position:" + position);
        this.changeMode(HOMING);
        checkEposMode(HOMING);
        this.writeParameter(HomePosition, position);
        this.writeParameter(HomingMethod, 35);
        this.enable();
        /* start homing */
        this.writeControlWord(0x1F);
        checkHomingDone();
    }
}

==> org-lsst-ccs-subsystem-fcs-main-5.5.3-sources.jar: org/lsst/ccs/subsystems/fcs/common/EPOSControllerForCarousel.java <==
package org.lsst.ccs.subsystems.fcs.common;

import org.lsst.ccs.command.annotations.Command;
import static org.lsst.ccs.subsystems.fcs.EPOSEnumerations.EposMode.HOMING;
import static org.lsst.ccs.subsystems.fcs.EPOSEnumerations.Parameter.HomePosition;
import static org.lsst.ccs.subsystems.fcs.EPOSEnumerations.Parameter.HomingMethod;
import static org.lsst.ccs.subsystems.fcs.EPOSEnumerations.Parameter.PositionSensorType;
import static org.lsst.ccs.subsystems.fcs.FCSCst.FCSLOG;
import static org.lsst.ccs.subsystems.fcs.FcsEnumerations.FcsAlert.HARDWARE_ERROR;
import org.lsst.ccs.subsystems.fcs.errors.FcsHardwareException;
import org.lsst.ccs.subsystems.fcs.utils.FcsUtils;

/**
 *
 * @author virieux
 */
public interface EPOSControllerForCarousel extends EPOSController {

    int getPositionSensorType();

    void setPositionSensorType(int pst);

    /**
     * For carousel, status word is read by PDO so no need to read by SDO. During
     * actions, read by PDO are done every 250ms.
     *
     * @return status word
     */
    @Override
    default boolean isTargetReached() {
        /* target is reached when bit10 of status word is 1 */
        return ((getStatusWord() >> 10) & 1) == 1;
    }

    /**
     * Set PositionSensorType to Absolute encoder SSI (value=4)
     */
    @Command(type = Command.CommandType.ACTION, level = Command.ENGINEERING1, description = "GoToSwitchOnDisabled and set PositionSensorType to Absolute encoder SSI : 4")
    default void setPositionSensorTypeEncoderSSI() {
        FCSLOG.info(getName() + " setting PositionSensorType Absolute encoder SSI");
        goToSwitchOnDisabled();
        doSetEncoderSSI();
    }

    default void doSetEncoderSSI() {
        writeParameter(PositionSensorType, 4);
        setPositionSensorType(4);
        /* check that position sensor type is SSI encoder. */
        checkPositionSensorType(4);
        int ssiPosition = readSSIPosition();
        if (Math.abs(ssiPosition) <= 1) {
            /* there is a bug in the controller in reading SSI position */
            /* switch back to Incremental Encoder */
            this.raiseWarning(HARDWARE_ERROR, " after setPositionSensorTypeEncoderSSI, ssiPosition = " + ssiPosition
                    + " we go back to TypeSinusIncrementalEncoder", getName());
            doSetSinusIncrementalEncoder();
            /* and then back again to Encoder SSI */
            doSetEncoderSSI();
        }
    }

    /**
     * Set PositionSensorType to Sinus Incremental Encoder(value=8)
     */
    @Command(type = Command.CommandType.ACTION, level = Command.ENGINEERING1, description = "GoToSwitchOnDisabled "
            + "and set PositionSensorType to Sinus Incremental Encoder : 8")
    default void setPositionSensorTypeSinusIncrementalEncoder() {
        // TODO homingDone = false
        FCSLOG.info(getName() + " setting PositionSensorType Sinus Incremental Encoder");
        goToSwitchOnDisabled();
        doSetSinusIncrementalEncoder();
    }

    default void doSetSinusIncrementalEncoder() {
        writeParameter(PositionSensorType, 8);
        setPositionSensorType(8);
        /* check that position sensor type is sinus incremental encoder. */
        checkPositionSensorType(8);
    }

    default void checkPositionSensorType(int sensortype) {
        long timeoutMillis = 500;
        final long timeStart = System.currentTimeMillis();
        long duration = 0;
        boolean ok = false;
        while (!ok && duration <= timeoutMillis) {
            duration = System.currentTimeMillis() - timeStart;
            FcsUtils.sleep(2, getName());
            ok = (sensortype == readParameter(PositionSensorType));
        }
        if (!ok) {
            String msg = getName() + " couldn't change position sensor type during time allocated of " + timeoutMillis
                    + " ms";
            FCSLOG.error(msg);
            throw new FcsHardwareException(msg);
        }
        setPositionSensorType(sensortype);
    }

    /**
     *
     * @param position
     */
    @Command(type = Command.CommandType.ACTION, level = Command.ENGINEERING3, description = "Define the actual position as position given as argument.")
    @Override
    default void defineAbsolutePosition(int position) {
        FCSLOG.debug(getName() + " Defining Absolute Position:" + position);
        this.changeMode(HOMING);

        // TODO Added at SLAC july 21 for tests. Should be removed eventually or be incorporated to a safer method
        if (HOMING != readMode()) {
            // second chance
            FcsUtils.sleep(500, "Controller change mode");
            this.changeMode(HOMING);
        }
        checkEposMode(HOMING);
        this.writeParameter(HomePosition, position);
        this.writeParameter(HomingMethod, 35);
        this.goToOperationEnable();
        /* start homing */
        this.writeControlWord(0x1F);
        checkHomingDone();
    }
}
